#!/usr/bin/perl

#
# Author: papanda@papa.to
# License: 3-clause BSD license
#

package searchutil;

use DateTime;

sub searchutil::convtimestr {
    my ($starttime) = @_;
    my $sdt = undef;
    if ($starttime) {
        if ($starttime =~ m!^([\d][\d][\d][\d])-([\d][\d])-([\d][\d])[\sT_\+]([\d][\d]):([\d][\d]):([\d][\d])!) {
            eval {
                $sdt = DateTime->now(time_zone=>'local');
                $sdt->set(year=>$1, month=>$2, day=>$3, hour=>$4, minute=>$5, second=>$6);
            };
            if ($@) {
                $sdt = undef;
            }
        }
    }
    return $sdt;
}

sub searchutil::convdaterange {
    my ($st, $maxrangedays) = @_;
    
    if (!$st) {
	die "invalid date";
    }
    my $maxsec = $maxrangedays * 86400;
    if ($st =~ m!([\d][\d])([\d][\d])([\d][\d])-([\d][\d])([\d][\d])([\d][\d])-([\d][\d])([\d][\d])([\d][\d])!) {
	my $sdt1 = DateTime->now(time_zone=>'local');
	my $edt1 = DateTime->now(time_zone=>'local');
	$sdt1->set( year=>2000 + $1, month=>$2, day=>$3, hour=>$4, minute=>$5, second=>$6);
	$edt1->set( year=>2000 + $1, month=>$2, day=>$3, hour=>$7, minute=>$8, second=>$9);
	if ($sdt1 > $edt1) {
	    $edt1->add( days=>1 );
	}
	if (($edt1->epoch - $sdt1->epoch) > $maxsec) {
	    die "invalid date";
	}
	return ($sdt1, $edt1);

    } elsif ($st =~ m!^(\d{2})(\d{2})(\d{2})(\d{2})-(\d{2})(\d{2})(\d{2})(\d{2})!) {
	my $sdt1 = DateTime->now( time_zone=>'local' );
	my $edt1 = DateTime->now( time_zone=>'local' );
	$sdt1->set( year=>2000 + $1, month=>$2, day=>$3, hour=>$4, minute=>0, second=> 0);
	$edt1->set( year=>2000 + $5, month=>$6, day=>$7, hour=>$8, minute=>0, second=> 0);
	if ($sdt1 > $edt1 || ($edt1->epoch - $sdt1->epoch) > $maxsec) {
	    die "invalid date";
	}
	return ($sdt1, $edt1);
    }
    die "invalid date";
}

sub searchutil::sqlite3date {
    my ($dt) = @_;
    return sprintf("%04d-%02d-%02d %02d:%02d:%02d", $dt->year, $dt->month, $dt->day, $dt->hour, $dt->minute, $dt->second);
}

sub searchutil::sqlite3rev {
    my ($str) = @_;
    my $dt = DateTime->now(time_zone=>'local');
    if ($str =~ m!([\d]+)-([\d]+)-([\d]+)\s([\d]+):([\d]+):([\d]+)!) {
	$dt->set(year=>$1, month=>$2, day=>$3, hour=>$4, minute=>$5, second=>$6);
    }
    return $dt;
}

1;
