#!/usr/bin/perl

#
# Author: papanda@papa.to
# License: 3-clause BSD license
#

use strict;
use CGI;
use DateTime;
use Unicode::Japanese;
use Time::HiRes qw( gettimeofday tv_interval );
use Data::Dumper;
use JSON;
use DBI;
use utf8;

require 'nibbleconf.pl';
require 'searchconf.pl';
require 'searchutil.pl';
require 'jsonutil.pl';
require 'wordutil.pl';

my $q = new CGI;

my @allparams = $q->param;
if (scalar @allparams > $searchconf::allparamsmax) {
    print &jsonutil::output($q, 406, '{ "error": "param is too long" }');
    exit 0;
}

my $now = time();
my $t0 = [gettimeofday];
my $op = {};
my $cachesec = $searchconf::cachesec;

# 文字列条件
my $sstr = $q->param('sstr') || 'none';
utf8::decode($sstr) if (!utf8::is_utf8($sstr));
$sstr =~ s![/,:<>()=~*`'"\|\?\$\[\]\"]+!!g; # 記号置換 \s . - @ # はOK
if (length $sstr > $searchconf::sstrlenmax) {
    print &jsonutil::output($q, 406, '{ "error": "invalid sstr" }');
    exit 0;
}
foreach my $word (split(/[\s]+/, $sstr)) {
    if (length $word > 1) {
	$sstr = $word;
	last;
    }
}
$op->{sstr} = $sstr;

# 物理チャンネル検索条件
my $phychsel = $q->param('phychsel') || -1;

# ジャンル検索条件
my $nibsel = $q->param('nibsel') || -1;

# ソート条件
my $vsorttype = $q->param('vsorttype') || 'wstartdesc';
if ($vsorttype eq 'wstartdesc') {
} elsif ($vsorttype eq 'wstartasc') {
} elsif ($vsorttype eq 'wscoredesc') {
} elsif ($vsorttype eq 'wscoreasc') {
} else {
    print &jsonutil::output($q, 406, '{ "error": "invalid vsorttype" }');
    exit 0;
}
my $vreqnum = $q->param('vreqnum') || $searchconf::vreqnumdef; 
$vreqnum = $searchconf::vreqnummax if ($vreqnum > $searchconf::vreqnummax);
my $voffset = $q->param('voffset') || 0;

# 日付指定
my $edt = DateTime->from_epoch( epoch=>$now, time_zone=>'local' );
my $tsexpiredays =$searchconf::tsexpiredays;
my $sdt = $edt->clone();
$sdt->subtract( days=> $tsexpiredays );
my $sdt0 = $sdt->clone();
if ($q->param('st')) {
    eval {
	($sdt, $edt) = &searchutil::convdaterange($q->param('st'), 7);
	if ($sdt0 > $sdt) {
	    $sdt = $sdt0->clone();
	}
    };
    if ($@) {
	print &jsonutil::output($q, 406, '{ "error": "invalid datestring(st)" }');
	exit 0;
    }
}
my $spandt = DateTime->now( time_zone=>'local' );
$spandt->subtract( days=>1 );

######################### INIT ######################################
my $dsn = 'DBI:SQLite:dbname='.$searchconf::dbpath;
my $dbh = DBI->connect($dsn, undef, undef, {RaiseError => 1, AutoCommit => 0});

$op->{seladdr} = '127.0.0.1';
$op->{etag} = 'sqlite3';
$op->{cached} = 0;
$op->{total} = 0;
$op->{offset} = 0;
$op->{num} = 0;

my $nstable = {};
my $chtable = {};
my $sql = "select dc_ch,dc_name,dc_network_id,dc_service_id from da_channel;";
my $sth = $dbh->prepare($sql);
my $rv = $sth->execute();
while (my $rcols = $sth->fetch()) {
    my ($dc_ch, $dc_name, $dc_network_id, $dc_service_id) = @$rcols;
    $nstable->{$dc_network_id . ':' . $dc_service_id} = {
	'phych'=>$dc_ch,
	'provider'=>$dc_name,
    };
    $chtable->{$dc_ch} = {
	'networkid'=>$dc_network_id,
	'serviceid'=>$dc_service_id,
	'provider'=>$dc_name,
    };
}

my $starttime = &searchutil::sqlite3date($sdt);
my $endtime = &searchutil::sqlite3date($edt);

my $epg = {};
if (1) {
    my $sql = 'select p.dv_id,p.dv_service_id,p.dv_original_network_id,p.dv_event_id,p.dv_start_time,datetime(p.dv_start_time, p.dv_duration),p.dv_name,p.dv_text, (select g.mg_level1 from da_genre g where g.dv_id = p.dv_id order by g.dg_id limit 1) from da_event p where p.dv_start_time >= ? and p.dv_start_time <= ?';
    if ($sstr eq 'none') {
	# don't care
    } else {
	$sql = $sql . " and (p.dv_name like '%" . $sstr . "%' or p.dv_text like '%" . $sstr . "%' )";
    }
    my $sth = $dbh->prepare($sql);
    my $rv = $sth->execute($starttime, $endtime);
    while (my $rcols = $sth->fetch()) {
	my ($dv_id, 
	    $dv_service_id,
	    $dv_original_network_id,
	    $dv_event_id,
	    $dv_start_time,
	    $dv_end_time,
	    $dv_name,
	    $dv_text,
	    $dv_nib) = @$rcols;
	my $stime1 = &searchutil::sqlite3rev($dv_start_time);
	my $etime1 = &searchutil::sqlite3rev($dv_end_time);
	next if (!exists $nstable->{$dv_original_network_id . ':' . $dv_service_id});
	my $phych = $nstable->{$dv_original_network_id . ':' . $dv_service_id}->{'phych'};
	my $provider = $nstable->{$dv_original_network_id . ':' . $dv_service_id}->{'provider'};
	utf8::decode($provider);
	my $epgid = $dv_original_network_id . ':' . $dv_service_id . ':' . $dv_event_id;
	if ($phychsel >= 13 && $phychsel < 300) {
	    next if ($phychsel != $phych);
	}	
	if ($nibsel >= 1 && $nibsel <= 16) {
	    my $nib = $nibsel - 1;
	    if (exists $nibbleconf::table{$nib} && exists $nibbleconf::table{$dv_nib}) {
		next if ($nib != $dv_nib);
	    }
	}
	my $vgenre = $nibbleconf::table{$dv_nib};
	utf8::decode($vgenre);
	my $vtitle = $dv_name;
	utf8::decode($vtitle);
	&wordutil::ctrlesc(\$vtitle);
	my $vdesc = $dv_text;
	utf8::decode($vdesc);
	&wordutil::ctrlesc(\$vdesc);

	$epg->{$epgid} = {
	    'wstart'=>sprintf("%d", $stime1->epoch),
	    'vscore'=>'0',
	    'wend'=>sprintf("%d", $etime1->epoch),
	    'vch'=>$provider,
	    'phych'=>sprintf("%d", $phych),
	    'nibble'=>sprintf("%d", $dv_nib),
	    'vgenre'=>$vgenre,
	    'networkid'=>sprintf("%d", $dv_original_network_id),
	    'serviceid'=>sprintf("%d", $dv_service_id),
	    'eventid'=>sprintf("%d", $dv_event_id),
	    'vtitle'=>$vtitle,
	    'vdesc'=>$vdesc,
	    'broken'=>'0',
	};
    }
}

########################### RESULT #############################
# 先に閉じておく
$dbh->disconnect();

my $total = scalar keys %$epg;
if ($voffset > $total) {
    $voffset = $total;
}
my $skip = $voffset;
my $num = 0;
my @result = ();

my @elist = ();
if ($vsorttype eq 'wstartdesc') {
    @elist = sort { $epg->{$b}->{wstart} <=> $epg->{$a}->{wstart} } keys %$epg;
} elsif ($vsorttype eq 'wstartasc') {
    @elist = sort { $epg->{$a}->{wstart} <=> $epg->{$b}->{wstart} } keys %$epg;
} elsif ($vsorttype eq 'wscoredesc') {
    @elist = sort { $epg->{$b}->{vscore} <=> $epg->{$a}->{vscore} } keys %$epg;
} elsif ($vsorttype eq 'wscoreasc') {
    @elist = sort { $epg->{$a}->{vscore} <=> $epg->{$b}->{vscore} } keys %$epg;
}

foreach my $epgid (@elist) {
    if ($skip > 0) {
	$skip--;
	next;
    }
    push(@result, $epg->{$epgid});
    $num++;
    if ($num >= $vreqnum) {
	last;
    }
}

$op->{total} = sprintf("%d", $total);
$op->{offset} = sprintf("%d", $voffset);
$op->{num} = sprintf("%d", $num);
$op->{result} = \@result;

############################## OUTPUT ######################################
# 出力
my $output = JSON->new->utf8(0)->encode($op);
utf8::encode($output); # UTF8フラグを外す
print &jsonutil::output($q, 200, $output, 'text/json', $cachesec);

1;
